window.Gleam.initSearch([{"doc":"rsvp","title":"rsvp","content":"# rsvp\n\n[![Package Version](https://img.shields.io/hexpm/v/rsvp)](https://hex.pm/packages/rsvp)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/rsvp/)\n\n```sh\ngleam add rsvp@1\n```\n\nRsvp helps you make HTTP requests and handle their responses in a Lustre application.\nThis library supports both client side SPAs and components, _and_ server components\nmeaning applications that use rsvp are more likely to be suitable as **universal\ncomponents**.\n\n## Usage\n\nRsvp separates the concerns of making HTTP requests from handling their responses\nmeaning we need to do three things to properly make a request:\n\n- First, we must define a [`Handler`](./rsvp.html#Handler) that describes how to\n  handler the response and turn it into a message your `update` function can\n  understand.\n\n- Second, we must create the [`Effect`](https://hexdocs.pm/lustre/lustre/effect.html)\n  that describes the request using either the [`get`](./rsvp.html#get) or\n  [`post`](./rsvp.html#post) convenience functions, or by constructing a\n  [`Request`](https://hexdocs.pm/gleam_http/gleam/http/request.html#Request) and\n  using [`send`](./rsvp.html#send) directly.\n\n- Finally, we must return that `Effect` in our application&#39;s `init` or `update`\n  functions so that the runtime can perform the request.\n\n## Recipes\n\nThe following recipes are available to help you get started with rsvp:\n\n- [Fetching JSON](./recipes/get-json.html)\n\n- [Post JSON](./recipes/post-json.html)\n\n- [Include authorization headers](./recipes/authorization-headers.html)\n\nIf you think a common use case is missing or you&#39;d like to contribute a recipe\nyourself, please open an issue or pull request!\n","url":"index.html"},{"doc":"rsvp","title":"rsvp","content":"","url":"#"},{"doc":"rsvp","title":"rsvp","content":"# Get JSON from an API\n\nRsvp it easy to fetch remote resources like JSON from an API using the\n[`get`](https://hexdocs.pm/rsvp/rsvp.html#get) effect.\n\n```gleam\nimport gleam/dynamic/decode\nimport gleam/int\nimport rsvp\n\ntype Post {\n  Post(user_id: Int, id: Int, title: String, completed: Bool)\n}\n\ntype Msg {\n  ApiReturnedPost(Result(Post, rsvp.Error))\n}\n\nfn get_todo(id) {\n  let url = &quot;https://jsonplaceholder.typicode.com/todos/&quot; &lt;&gt; int.to_string(id)\n  let handler = rsvp.expect_json(decode_todo(), ApiReturnedPost)\n\n  rsvp.get(url, handler)\n}\n\nfn decode_todo() {\n  use user_id &lt;- decode.field(&quot;userId&quot;, decode.int)\n  use id &lt;- decode.field(&quot;id&quot;, decode.int)\n  use title &lt;- decode.field(&quot;title&quot;, decode.string)\n  use completed &lt;- decode.field(&quot;completed&quot;, decode.bool)\n\n  decode.success(Post(user_id:, id:, title:, completed:))\n}\n```\n\n&gt; **Note**: remember that rsvp returns a Lustre\n&gt; [`Effect`](https://hexdocs.pm/lustre/lustre/effect.html#Effect) for the runtime\n&gt; to perform. For the request to be made, you need to return the effect from your\n&gt; application&#39;s `init` or `update` function!\n","url":"recipes/get-json.html"},{"doc":"rsvp","title":"rsvp","content":"# Post JSON to an API\n\nRsvp makes it simple to send JSON data to an API using the\n[`post`](https://hexdocs.pm/rsvp/rsvp.html#post) effect.\n\n```gleam\nimport gleam/dynamic/decode\nimport gleam/http/response.{type Response}\nimport gleam/json\nimport rsvp\n\ntype NewPost {\n  NewPost(title: String, body: String, user_id: Int)\n}\n\ntype Msg {\n  ApiCreatedPost(Result(Response(String), rsvp.Error))\n}\n\nfn create_post(new_post: NewPost) {\n  let body = json.object([\n    #(&quot;title&quot;, json.string(new_post.title)),\n    #(&quot;body&quot;, json.string(new_post.body)),\n    #(&quot;userId&quot;, json.int(new_post.user_id)),\n  ])\n\n  let handler = rsvp.expect_ok_response(ApiCreatedPost)\n\n  rsvp.post(&quot;https://jsonplaceholder.typicode.com/posts&quot;, body, handler)\n}\n\n// Decoder for the API response\nfn decode_post() {\n  use id &lt;- decode.field(&quot;id&quot;, decode.int)\n  use title &lt;- decode.field(&quot;title&quot;, decode.string)\n  use body &lt;- decode.field(&quot;body&quot;, decode.string)\n  use user_id &lt;- decode.field(&quot;userId&quot;, decode.int)\n\n  decode.success(Post(id:, title:, body:, user_id:))\n}\n```\n\n&gt; **Note**: Like with GET requests, remember that rsvp returns a Lustre\n&gt; [`Effect`](https://hexdocs.pm/lustre/lustre/effect.html#Effect) that needs to be\n&gt; returned from your application&#39;s `init` or `update` function to be executed!\n","url":"recipes/post-json.html"},{"doc":"rsvp","title":"rsvp","content":"# Including authorization headers in your requests\n\nThe [`get`](https://hexdocs.pm/rsvp/rsvp.html#get) and\n[`post`](https://hexdocs.pm/rsvp/rsvp.html#post) convenience functions do not\ngive you any control over the HTTP request being sent. To include HTTP headers\nlike authorization tokens, you need to construct the [`Request`](https://hexdocs.pm/gleam_http/gleam/http/request.html#Request)\nyourself and use the [`send`](https://hexdocs.pm/rsvp/rsvp.html#send) function.\n\n```gleam\nimport gleam/dynamic/decode\nimport gleam/http.{Get}\nimport gleam/http/request\nimport rsvp\n\ntype User {\n  User(username: String, email: String)\n}\n\ntype Msg {\n  ApiReturnedUser(Result(User, rsvp.Error))\n}\n\nfn get_user(api_token: String) {\n  let request =\n    request.new()\n    |&gt; request.set_method(Get)\n    |&gt; request.set_header(&quot;authorization&quot;, &quot;Bearer &quot; &lt;&gt; api_token)\n    |&gt; request.set_host(&quot;api.example.com&quot;)\n    |&gt; request.set_path(&quot;/user&quot;)\n\n  let handler = rsvp.expect_json(decode_user(), ApiReturnedUser)\n\n  rsvp.send(request, handler)\n}\n\nfn decode_user() {\n  use username &lt;- decode.field(&quot;username&quot;, decode.string)\n  use email &lt;- decode.field(&quot;email&quot;, decode.string)\n\n  decode.success(User(username:, email:))\n}\n```\n\n&gt; **Note**: remember that rsvp returns a Lustre\n&gt; [`Effect`](https://hexdocs.pm/lustre/lustre/effect.html#Effect) for the runtime\n&gt; to perform. For the request to be made, you need to return the effect from your\n&gt; application&#39;s `init` or `update` function!\n","url":"recipes/authorization-headers.html"},{"doc":"rsvp","title":"Error","content":"pub type Error {\n  BadBody\n  BadUrl(String)\n  HttpError(Response(String))\n  JsonError(json.DecodeError)\n  NetworkError\n  UnhandledResponse(Response(String))\n}\n A request might fail for a number of reasons. This type is a high-level\n wrapper over the different kinds of errors that might occur when creating and\n executing an HTTP request.\n\nBadBody\n This error can happen when we successfully receive an HTTP response but the\n body of the response is invalid or not well-formed.\n\n\nBadUrl(String)\n This error can happen when the URL string provided to the [`get`](#get) or\n [`post`](#post) helpers is not well-formed.\n\n\nHttpError(Response(String))\n This error can happen when the HTTP response status code is not in the `2xx`\n range but a handler expected it to be.\n\n\nJsonError(json.DecodeError)\n This error is returned when decoding a JSON response body fails.\n\n\nNetworkError\n This error can happen when the HTTP request fails to connect to the server\n or there is some other connectivity issue.\n\n\nUnhandledResponse(Response(String))\n This error can be returned by a handler when it does not know how to handle\n a response. For example, the [`expect_json`](#expect_json) handler will return\n this error if the response content-type is not `&quot;application/json&quot;`.\n\n\nSynonyms:\nrsvp.Error\nrsvp Error","url":"rsvp.html#Error"},{"doc":"rsvp","title":"Handler","content":"pub opaque type Handler(msg)\n A handler is a function that knows how to take the result of an HTTP request\n and turn it into a message that can be dispatched back to your `update`\n function. rsvp exposess a number of handlers for common scenarios:\n\n - [`expect_json`](#expect_json) to ensure a response&#39;s content-type is\n   `&quot;application/json&quot;` and run a JSON decoder on that body.\n\n - [`expect_text`](#expect_text) to ensure a response&#39;s content-type specifies\n   `&quot;text/&quot;` and return the body as a string.\n\n - [`expect_ok_response`](#expect_ok_response) to handle any response with a\n   2xx status code.\n\n - [`expect_any_response`](#expect_any_response) to handle any HTTP response,\n   including 4xx and 5xx errors.\n\n\nSynonyms:\nrsvp.Handler\nrsvp Handler","url":"rsvp.html#Handler"},{"doc":"rsvp","title":"expect_any_response","content":"pub fn expect_any_response(\n  handler: fn(Result(Response(String), Error)) -&gt; a,\n) -&gt; Handler(a)\n Handle any HTTP response, regardless of status code. Your custom handler will\n still have to handle potential errors such as network errors or malformed\n responses.\n\n It is uncommon to need a handler this low-level, instead you can consider the\n following more-specific handlers:\n\n - [`expect_ok_response`](#expect_ok_response) to handle any response with a\n   `2xx` status code.\n\n - [`expect_json`](#expect_json) to handle responses from JSON apis\n\nSynonyms:\nrsvp.expect_any_response\nrsvp expect_any_response","url":"rsvp.html#expect_any_response"},{"doc":"rsvp","title":"expect_json","content":"pub fn expect_json(\n  decoder: Decoder(a),\n  handler: fn(Result(a, Error)) -&gt; b,\n) -&gt; Handler(b)\n A handler that runs a JSON decoder on a response body and returns the result\n as a message. This handler will check the following conditions:\n\n - The response status code is `2xx`.\n\n - The response content-type is `&quot;application/json&quot;`\n\n - The response body can be decoded using the provided JSON decoder\n\n If any of these conditions are not met, an `Error` will be returned instead.\n The specific error will depend on which condition failed:\n\n - `4xx` and `5xx` status codes will return `HttpError`\n\n - Other non `2xx` status codes will return `UnhandledResponse`\n\n - A missing or incorrect `content-type` header will return `UnhandledResponse`\n\n - A JSON decoding error will return `JsonError`\n\n **Note**: if you need more advanced handling of the request body directly, you\n should use the more-general [`expect_ok_response`](#expect_ok_response) or\n [`expect_any_response`](#expect_any_response) handlers.\n\nSynonyms:\nrsvp.expect_json\nrsvp expect_json","url":"rsvp.html#expect_json"},{"doc":"rsvp","title":"expect_ok_response","content":"pub fn expect_ok_response(\n  handler: fn(Result(Response(String), Error)) -&gt; a,\n) -&gt; Handler(a)\n Handle any response with a `2xx` status code. This handler will return an\n `Error` if the response status code is not in the `2xx` range. The specific\n error will depend on the status code:\n\n - `4xx` and `5xx` status codes will return `HttpError`\n\n - Other non `2xx` status codes will return `UnhandledResponse`\n\n **Note**: if you need to handle HTTP responses with different status codes,\n you should use the more-general [`expect_any_response`](#expect_any_response)\n handler.\n\nSynonyms:\nrsvp.expect_ok_response\nrsvp expect_ok_response","url":"rsvp.html#expect_ok_response"},{"doc":"rsvp","title":"expect_text","content":"pub fn expect_text(\n  handler: fn(Result(String, Error)) -&gt; a,\n) -&gt; Handler(a)\n Handle the body of a plain text response. This handler will check the\n following conditions:\n\n - The response status code is `2xx`.\n\n - The response content-type specifies `&quot;text/&quot;` such as `&quot;text/plain&quot;` or\n   `&quot;text/html&quot;`.\n\n If any of these conditions are not met, an `Error` will be returned instead.\n The specific error will depend on which condition failed:\n\n - `4xx` and `5xx` status codes will return `HttpError`\n\n - Other non `2xx` status codes will return `UnhandledResponse`\n\n - A missing or incorrect `content-type` header will return `UnhandledResponse`\n\n **Note**: if you need more advanced handling of the request body directly, you\n should use the more-general [`expect_ok_response`](#expect_ok_response) or\n [`expect_any_response`](#expect_any_response) handlers.\n\nSynonyms:\nrsvp.expect_text\nrsvp expect_text","url":"rsvp.html#expect_text"},{"doc":"rsvp","title":"get","content":"pub fn get(url: String, handler: Handler(a)) -&gt; Effect(a)\n A convenience function to send a `GET` request to a URL and handle the response\n using a [`Hander`](#Handler).\n\n **Note**: if you need more control over the kind of request being sent, for\n example to set additional headers or use a different HTTP method, you should\n use the more-general [`send`](#send) function insteaed.\n\n **Note**: On the **JavaScript** target this will use the `fetch` API. Make\n sure you have a polyfill for it if you need to support older browsers or\n server-side runtimes that don&#39;t have it.\n\n **Note**: On the **Erlang** target this will use the `httpc` module. Each\n request will start a new linked process to make and handle the request.\n\nSynonyms:\nrsvp.get\nrsvp get","url":"rsvp.html#get"},{"doc":"rsvp","title":"parse_relative_uri","content":"pub fn parse_relative_uri(uri_string: String) -&gt; Result(Uri, Nil)\n The standard library [`uri.parse`](https://hexdocs.pm/gleam_stdlib/0.45.0/gleam/uri.html#parse)\n function does not support relative URIs. When running in the browser, however,\n we have enough information to resolve relative URIs into complete ones!\n\n This function will always fail when running on the server, but in the browser\n it will resolve relative URIs based on the current page&#39;s URL\n\nSynonyms:\nrsvp.parse_relative_uri\nrsvp parse_relative_uri","url":"rsvp.html#parse_relative_uri"},{"doc":"rsvp","title":"post","content":"pub fn post(\n  url: String,\n  body: Json,\n  handler: Handler(a),\n) -&gt; Effect(a)\n A convenience function for sending a POST request with a JSON body and handle\n the response with a handler function. This will automatically set the\n `content-type` header to `application/json` and handle requests to relative\n URLs if this effect is running in a browser.\n\n **Note**: if you need more control over the kind of request being sent, for\n example to set additional headers or use a different HTTP method, you should\n use the more-general [`send`](#send) function insteaed.\n\n **Note**: On the **JavaScript** target this will use the `fetch` API. Make\n sure you have a polyfill for it if you need to support older browsers or\n server-side runtimes that don&#39;t have it.\n\n **Note**: On the **Erlang** target this will use the `httpc` module. Each\n request will start a new linked process to make and handle the request.\n\nSynonyms:\nrsvp.post\nrsvp post","url":"rsvp.html#post"},{"doc":"rsvp","title":"send","content":"pub fn send(\n  request: Request(String),\n  handler: Handler(a),\n) -&gt; Effect(a)\n Send a [`Request`](https://hexdocs.pm/gleam_http/gleam/http/request.html#Request)\n and dispatch a message back to your `update` function when the response is\n handled.\n\n For simple requests, you can use the more-convenient [`get`](#get) and\n [`post`](#post) functions instead.\n\n **Note**: On the **JavaScript** target this will use the `fetch` API. Make\n sure you have a polyfill for it if you need to support older browsers or\n server-side runtimes that don&#39;t have it.\n\n **Note**: On the **Erlang** target this will use the `httpc` module. Each\n request will start a new linked process to make and handle the request.\n\nSynonyms:\nrsvp.send\nrsvp send","url":"rsvp.html#send"},{"doc":"rsvp","title":"rsvp","content":"","url":"rsvp.html"}]);